/**
 * 
 *    Copyright 2013 EOSSOnline Limited
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *  
**/
package com.eossonline.edrive.events;

import java.nio.file.Path;
import java.util.EventObject;

import org.apache.chemistry.opencmis.commons.impl.json.JSONObject;

import com.eossonline.esync.ESync;



public class EDriveEvent extends EventObject {

	private static final long serialVersionUID = 1L;
	
	private String action;
	private String direction;
	private String stage;
	private Path location;
	private long eventTimeMillis;
	private long size;
	private Throwable exception;
	
	public EDriveEvent (
			ESync source,		// The source of the event i.e. which synchroniser
			String action,		// The action - Create, Update, Delete, Sync
			String direction,	// The direction the action is occurring - OUTBOUND (to Repository), INBOUND (from Repository)
			String stage,		// Is this event generated at the beginning or the end - STARTED, FINISHED, CANCEL
			Path location,		// What Folder or Document is this event associated with
			long size,			// the size of files synchronised 
			Throwable exception	// Did an error occur. A null means all is well
		) {
		super(source);
		this.action = action;
		this.direction = direction;
		this.size = size;
		this.stage = stage;
		this.location = location;
		this.exception = exception;
		this.eventTimeMillis = System.currentTimeMillis();
	}
	
	public ESync getSource() { return (ESync)super.getSource(); }
	public String getAction() { return action; }
	public String detDirection() { return direction; }
	public String getStage() { return stage; }
	public Path getLocation() { return location; }
	public long getEventTimeMillis() { return eventTimeMillis; }
	public long getSize() { return size; }
	public Throwable getException() { return exception; }
	
	public String toString() {
		JSONObject obj = new JSONObject();
		obj.put("name", getSource().getName());
		obj.put("action", action);
		obj.put("direction", direction);
		obj.put("stage", stage);
		obj.put("location", location.toString());
		obj.put("size", size);
		obj.put("eventTimeMillis", eventTimeMillis);
		obj.put("exception", exception);
		
		return obj.toJSONString();
	}
}
