/**
 * 
 *    Copyright 2013 EOSSOnline Limited
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *  
**/
package com.eossonline.edrive.service;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.InetAddress;
import java.net.MulticastSocket;
import java.net.UnknownHostException;
import java.util.concurrent.ArrayBlockingQueue;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.eossonline.edrive.events.EDriveEvent;

public class EDriveMulticastServer extends Thread {
	
	private ArrayBlockingQueue<EDriveEvent> queue = new ArrayBlockingQueue<EDriveEvent>(100);
	private InetAddress ia = null;
	private int port = 0;
	private MulticastSocket ms = null;
	
	
	private static final Logger logger = LoggerFactory.getLogger(EDriveMulticastServer.class);

	public EDriveMulticastServer(EDrive eDrive, String multicastIP, int port) {

		try {
			ia = InetAddress.getByName(multicastIP);
			ms = new MulticastSocket(port);
			this.port = ms.getLocalPort();
			ms.setTimeToLive(1);
			ms.joinGroup(ia);

			start();

		} catch (UnknownHostException e) {
			logger.error("Failed to configure Multicast Server on IP [{}] and port [{}]", multicastIP, port, e);
			throw new RuntimeException(e);
		} catch (IOException ioe) {
			logger.error("Failed to configure Multicast Server on IP [{}] and port [{}]", multicastIP, port, ioe);
			throw new RuntimeException(ioe);
		}
	}
	
	public void shutdown() {
		logger.warn("Terminating...");
		this.interrupt();
		logger.warn("Terminated...");
	}
	
	public void run() {
		
		logger.info("Multicast configured on IP [{}] and port [{}]...", ia.getHostAddress(), this.port);
		
		DatagramPacket dp = null;
		try {
			EDriveEvent event = null;
			while(!Thread.currentThread().isInterrupted()) {
				try {
					event = queue.take();
					byte [] data = event.toString().getBytes();
					dp = new DatagramPacket(data, data.length, ia, port);
					ms.send(dp);
				} catch (InterruptedException ie) {
					break;
				}
			}
			ms.leaveGroup(ia);
			ms.close();
			
		}  catch(Exception e) {
			logger.error("Exception while running MulticastServer. Terminating...", e);
		}
	}
	
	protected void addEvent(EDriveEvent event) {
		queue.add(event);
	}
}